/* ----------------------------------------------
 *   FAT study code
 *         KAZ.Imamura
 * ---------------------------------------------- */
 
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <error.h>

// ------------------------------------------------------
//  Definitions
// ------------------------------------------------------

#define MAX_FILE_IN_DIR		128

#define DIR_ATTR_READONLY			0x01
#define DIR_ATTR_HIDDEN				0x02
#define DIR_ATTR_SYSTEM				0x04
#define DIR_ATTR_VOLUME_ID			0x08
#define DIR_ATTR_DIRECTORY			0x10
#define DIR_ATTR_ARCHIVE			0x20
#define DIR_ATTR_LONG_NAME			(DIR_ATTR_READONLY|DIR_ATTR_HIDDEN|DIR_ATTR_SYSTEM|DIR_ATTR_VOLUME_ID)

// BPB information
typedef struct BPB_information {
	unsigned int  ResvdSecCnt;				// Offset 24 Size 2  
	unsigned int  BytsPerSec;				// Offset 11 Size 2
	unsigned char NumFATs;					// Offset 16 Size 1
	unsigned int  RootEntCnt;				// Offset 17 Size 2 (Only FAT16 has valid data, FAT32 has 0.)
	unsigned int  FATSz16;					// Offset 22 Size 2 FAT size for FAT16
	unsigned int  TotSec16;					// Offset 19 Size 2
	unsigned long TotSec32;					// Offset 32 Size 4
	unsigned char SecPerClus;				// Offset 13 Size 1
											// Only for FAT32
	unsigned long FATSz32;					// Offset 36 Size 2 FAT size for FAT32
	unsigned long RootClus;					// Offset 44 Size 4
} BPB_INFO, *P_BPB_INFO;

// Keep file list on current directory
typedef struct {
	char			Name[13];
	unsigned long	FirstCluster;
	unsigned long	FileSize;
	unsigned char	Attributes;
} DIR_INFO, *P_DIR_INFO;

// Search status for DIR information
typedef struct {
	unsigned long	RootCluster;			// In case of static table, it should be 0.
	unsigned long	CurrentCluster;			// In case of static table, it should be 0.
	unsigned long	CurrentSector;
	unsigned int	ByteOffset;
} DIR_ENTRY, *P_DIR_ENTRY;

// Directory status
enum dir_status{
	DIR_ENTRY_EOD,
	DIR_ENTRY_EMPTY,
	DIR_LONG_FNAME,
	DIR_ENTRY_READY,
};

// ------------------------------------------------------
//  Prototype Definitions
// ------------------------------------------------------
void read_BPB( int fd );
void read_DIR( int fd );
void sort_DIR( int fd, P_DIR_INFO pDir );
unsigned char init_dir_entry( int fd, P_DIR_ENTRY pDirEnt, unsigned long cluster );
unsigned char get_dir_entry( int fd, P_DIR_ENTRY pDirEnt, P_DIR_INFO pDirInfo );
unsigned char next_dir_entry( int fd, P_DIR_ENTRY pDirEnt );
unsigned char read_file( int fd, P_DIR_INFO pDirInfo, unsigned int index );

unsigned char move_window ( int fd, unsigned long sector );
unsigned long Cluster2Sector( unsigned long cluster );
int GetNextCluster(int fd, unsigned long *p_cluster );

// ------------------------------------------------------
//  Variables
// ------------------------------------------------------
BPB_INFO BPB_info;						// BPB information
DIR_INFO dir_info[MAX_FILE_IN_DIR];		// Keep file list on current directory
DIR_ENTRY dir_entry;					// Search status for DIR information

unsigned char buf[512];					// Window buffer

unsigned long RootDirSectors;
unsigned long FATSectors;
unsigned long FirstDataSector;
unsigned long TotalSect;
unsigned long DataSectors;
unsigned long CountofClusters;

unsigned long CurrentWindowSector;
unsigned int  TotalFileNum;
unsigned char IsFAT32;

// ------------------------------------------------------
//  Read DIR function
// ------------------------------------------------------
void read_DIR( int fd ) {
	unsigned long SearchSector, SearchCluster;
	int i,a;
	unsigned char res;
	
	TotalFileNum = 0;
	while( TotalFileNum < MAX_FILE_IN_DIR ) {
		res = get_dir_entry(fd, &dir_entry, &dir_info[TotalFileNum]);
		if ( res == DIR_ENTRY_EOD ) break;
		else if	 ( res == DIR_ENTRY_READY ) TotalFileNum++;
		
		next_dir_entry(fd,  &dir_entry);
	}
	
	for(i=0; i<TotalFileNum; i++) {
		if(dir_info[i].Attributes & DIR_ATTR_DIRECTORY)
			printf("[read_DIR] [%12s]  0x%08X %12d bytes\r\n", dir_info[i].Name, dir_info[i].FirstCluster, dir_info[i].FileSize );
		else
			printf("[read_DIR]  %12s   0x%08X %12d bytes\r\n", dir_info[i].Name, dir_info[i].FirstCluster, dir_info[i].FileSize );
	}
}


// ------------------------------------------------------
//  DIR entry initialize (with move window)
// ------------------------------------------------------
unsigned char init_dir_entry( int fd, P_DIR_ENTRY pDirEnt, unsigned long cluster ) {
	unsigned char res;
	
	if( cluster == 0 ) {			// Root directory ?
		if(IsFAT32) {
			pDirEnt->RootCluster   = 2;
			pDirEnt->CurrentSector = FirstDataSector;
		} else {
			pDirEnt->RootCluster   = 0;
			pDirEnt->CurrentSector = BPB_info.ResvdSecCnt + (BPB_info.NumFATs * FATSectors);
		}
	} else {
		pDirEnt->RootCluster   = cluster;
		pDirEnt->CurrentSector = Cluster2Sector( cluster );
	}
	pDirEnt->ByteOffset = 0;
	pDirEnt->CurrentCluster = pDirEnt->RootCluster;
	
	printf("[init_dir_entry] pDirEnt->RootCluster    : 0x%08X.\r\n", pDirEnt->RootCluster);
	printf("[init_dir_entry] pDirEnt->CurrentCluster : 0x%08X.\r\n", pDirEnt->CurrentCluster);
	printf("[init_dir_entry] pDirEnt->CurrentSector  : 0x%08X.\r\n", pDirEnt->CurrentSector);
	printf("[init_dir_entry] pDirEnt->ByteOffset     : 0x%08X.\r\n", pDirEnt->ByteOffset);
	res = move_window(fd, pDirEnt->CurrentSector);
	printf("[init_dir_entry] end\r\n");
	return res;
}

// ------------------------------------------------------
//  DIR go next (with move window)
// ------------------------------------------------------
unsigned char next_dir_entry( int fd, P_DIR_ENTRY pDirEnt ) {
	int res;
	
	pDirEnt->ByteOffset += 32;			// DIR entry is 32 byte
	if( pDirEnt->ByteOffset >= BPB_info.BytsPerSec ) { // sector changed ?
		pDirEnt->ByteOffset = 0;
		pDirEnt->CurrentSector++;
		if( pDirEnt->CurrentSector % BPB_info.SecPerClus == 0 ) {	// cluster changed?
			if( pDirEnt->RootCluster != 0 ) {
				// get next cluster
				GetNextCluster(fd, &pDirEnt->CurrentCluster);
				pDirEnt->CurrentSector = Cluster2Sector( pDirEnt->CurrentCluster );
			}
		}
	res = move_window(fd, pDirEnt->CurrentSector);
	printf("[next_dir_entry] end");
	} 
}

// ------------------------------------------------------
//  Get DIR entry (Copy to pDirInfo)
// ------------------------------------------------------
unsigned char get_dir_entry( int fd, P_DIR_ENTRY pDirEnt, P_DIR_INFO pDirInfo ) {
//	printf("[get_dir_entry] start.\r\n");
	if(buf[pDirEnt->ByteOffset+  0] == 0x00) 					return DIR_ENTRY_EOD;		// end of entries
	else if (buf[pDirEnt->ByteOffset+  0] == 0xE5)				return DIR_ENTRY_EMPTY;
	else if ( (buf[pDirEnt->ByteOffset+ 11] & 0x0F) == DIR_ATTR_LONG_NAME)	return DIR_LONG_FNAME;
	else {
		pDirInfo->FirstCluster =
						  buf[pDirEnt->ByteOffset+ 20] * 0x10000
						+ buf[pDirEnt->ByteOffset+ 21] * 0x1000000
						+ buf[pDirEnt->ByteOffset+ 26]
						+ buf[pDirEnt->ByteOffset+ 27] * 0x100;
		pDirInfo->Attributes = buf[pDirEnt->ByteOffset+ 11];
		pDirInfo->FileSize =
						  buf[pDirEnt->ByteOffset+ 28] 
						+ buf[pDirEnt->ByteOffset+ 29] * 0x100
						+ buf[pDirEnt->ByteOffset+ 30] * 0x10000;
						+ buf[pDirEnt->ByteOffset+ 31] * 0x1000000;
		
		pDirInfo->Name[ 0] = buf[pDirEnt->ByteOffset+  0];	pDirInfo->Name[ 1] = buf[pDirEnt->ByteOffset+  1];
		pDirInfo->Name[ 2] = buf[pDirEnt->ByteOffset+  2];	pDirInfo->Name[ 3] = buf[pDirEnt->ByteOffset+  3];
		pDirInfo->Name[ 4] = buf[pDirEnt->ByteOffset+  4];	pDirInfo->Name[ 5] = buf[pDirEnt->ByteOffset+  5];
		pDirInfo->Name[ 6] = buf[pDirEnt->ByteOffset+  6];	pDirInfo->Name[ 7] = buf[pDirEnt->ByteOffset+  7];
		
		if(pDirInfo->Attributes & DIR_ATTR_DIRECTORY) {
			pDirInfo->Name[ 8] = buf[pDirEnt->ByteOffset+  8];
			pDirInfo->Name[ 9] = buf[pDirEnt->ByteOffset+  9];
			pDirInfo->Name[10] = buf[pDirEnt->ByteOffset+ 10];
			pDirInfo->Name[11] = ' ';
			pDirInfo->Name[12] = 0x00;
			printf(" [%12s]  0x%08X\r\n", pDirInfo->Name, pDirInfo->FirstCluster );
		} else {
			pDirInfo->Name[ 8] = '.';
			pDirInfo->Name[ 9] = buf[pDirEnt->ByteOffset+  8];
			pDirInfo->Name[10] = buf[pDirEnt->ByteOffset+  9];
			pDirInfo->Name[11] = buf[pDirEnt->ByteOffset+ 10];
			pDirInfo->Name[12] = 0x00;
			printf("  %12s   0x%08X\r\n", pDirInfo->Name, pDirInfo->FirstCluster );
		}
		return  DIR_ENTRY_READY;
	}
}

// ------------------------------------------------------
//  Move window
// ------------------------------------------------------
unsigned char move_window ( int fd, unsigned long sector ) {
	if( CurrentWindowSector == sector ) return 0;
	printf("[move_window] Seek to 0x%08X.\r\n", sector);
	if( lseek(fd, (off_t) (sector * BPB_info.BytsPerSec), SEEK_SET) < 0 ) {
		printf("[move_window] Seek failed.\r\n");
		perror("[move_window]");
		return 0xFF;
	}
	printf("[move_window] Seek OK.\r\n");
	if( read(fd, buf, sizeof(buf)) == 0 ) {
		printf("[move_window] Read failed.\r\n");
		return 0xFF;
	}
	printf("[move_window] Read OK.\r\n");
	return 0;
}


// ------------------------------------------------------
//  Sort DIR
// ------------------------------------------------------
void sort_DIR( int fd, P_DIR_INFO pDir ) {
	int i,a;
	int dir_count;
	printf("[sort_DIR] *** BEFORE ***\r\n");
	for(i=0; i<TotalFileNum; i++) {
		if(pDir[i].Attributes & DIR_ATTR_DIRECTORY)
			printf("[sort_DIR] [%12s]  0x%08X %12d bytes\r\n", pDir[i].Name, pDir[i].FirstCluster, pDir[i].FileSize );
		else
			printf("[sort_DIR]  %12s   0x%08X %12d bytes\r\n", pDir[i].Name, pDir[i].FirstCluster, pDir[i].FileSize );
	}
	
	dir_count = 0;
	for(i=0; i<TotalFileNum; i++) {
		if( pDir[i].Attributes & DIR_ATTR_DIRECTORY ) {
			if( i > dir_count ) {
				printf("[sort_DIR] swap pDir[%d] <--> pDir[%d]\r\n", i, dir_count);
				// Swap
				DIR_INFO temp;
				for(a=0;a<sizeof(temp.Name);a++) temp.Name[a] = pDir[i].Name[a];
				temp.FirstCluster = pDir[i].FirstCluster;
				temp.FileSize = pDir[i].FileSize;
				temp.Attributes  = pDir[i].Attributes;
				
				for(a=0;a<sizeof(temp.Name);a++) pDir[i].Name[a] = pDir[dir_count].Name[a];
				pDir[i].FirstCluster = pDir[dir_count].FirstCluster;
				pDir[i].FileSize = pDir[dir_count].FileSize;
				pDir[i].Attributes  = pDir[dir_count].Attributes;
				
				for(a=0;a<sizeof(temp.Name);a++) pDir[dir_count].Name[a] = temp.Name[a];
				pDir[dir_count].FirstCluster = temp.FirstCluster;
				pDir[dir_count].FileSize = temp.FileSize;
				pDir[dir_count].Attributes  = temp.Attributes;
			}
			dir_count++;
		}
	}

	printf("[sort_DIR] *** AFTER ***\r\n");
	for(i=0; i<TotalFileNum; i++) {
		if(pDir[i].Attributes & DIR_ATTR_DIRECTORY)
			printf("[sort_DIR] [%12s]  0x%08X %12d bytes\r\n", pDir[i].Name, pDir[i].FirstCluster, pDir[i].FileSize );
		else
			printf("[sort_DIR]  %12s   0x%08X %12d bytes\r\n", pDir[i].Name, pDir[i].FirstCluster, pDir[i].FileSize );
	}
}

// ------------------------------------------------------
//  Cluster to Sector convert
// ------------------------------------------------------
unsigned long Cluster2Sector( unsigned long cluster ) {
	return ( FirstDataSector + (cluster - 2) * BPB_info.SecPerClus );
}

// ------------------------------------------------------
//  Check next cluster on the chain
// ------------------------------------------------------
int GetNextCluster(int fd, unsigned long *p_cluster ) {
	unsigned long FATSecNum, FATEntOffset;
	unsigned long EntryVal;
	
	if( BPB_info.BytsPerSec == 0 ) return -1;
	
	if(IsFAT32) {
		FATSecNum = BPB_info.ResvdSecCnt + (*p_cluster * 4 / BPB_info.BytsPerSec);
		FATEntOffset = (*p_cluster * 4) % BPB_info.BytsPerSec;
	} else {
		FATSecNum = BPB_info.ResvdSecCnt + (*p_cluster * 2 / BPB_info.BytsPerSec);
		FATEntOffset = (*p_cluster * 2) % BPB_info.BytsPerSec;
	}
	printf("[GetNextCluster] Current cluster: 0x%08X\r\n",*p_cluster);
	printf("[GetNextCluster] FATSecNum:    0x%08X\r\n", FATSecNum);
	printf("[GetNextCluster] FATEntOffset: 0x%08X\r\n", FATEntOffset);

	printf("[GetNextCluster] Seek to 0x%08X.\r\n", FATSecNum);
	if( lseek(fd, (off_t) (FATSecNum * BPB_info.BytsPerSec), SEEK_SET) < 0 ) {
		printf("[GetNextCluster] Seek failed.\r\n");
		perror("[GetNextCluster]");
		return;
	}
	if( read(fd, buf, sizeof(buf)) == 0 ) {
		printf("[GetNextCluster] Read failed.\r\n");
		perror("[GetNextCluster]");
		return;
	}
	
	if(IsFAT32) {
		EntryVal = *(unsigned long*) &buf[FATEntOffset] & 0x0FFFFFFF;
	} else {
		EntryVal = *(unsigned short*) &buf[FATEntOffset];
	}
	printf("[GetNextCluster] EntryVal: 0x%08X\r\n", EntryVal);
	
	if( (EntryVal > 0x00000002) && (EntryVal < 0x0FFFFFF6) ) {
		*p_cluster = EntryVal;
		printf("[GetNextCluster] next cluster: 0x%08X\r\n", *p_cluster);
		return 0;
	} else if ( (EntryVal > 0x0FFFFFF8) && (EntryVal < 0x0FFFFFFF) ) {
		// EOC
		printf("[GetNextCluster] it should be EOC.\r\n");
		return 1;
	} else if ( EntryVal == 0x0FFFFFF7 ) {
		printf("[GetNextCluster] it should be error cluster.\r\n");
		return -1;
	} else {
		printf("[GetNextCluster] it should be reserved cluster.\r\n");
		return -1;
	}
}


// ------------------------------------------------------
//  Read BPB information
// ------------------------------------------------------
void read_BPB(int fd) {
	int i;
	
	if( lseek(fd, 0L, SEEK_SET) < 0 ) {
		printf("[read_BPB] Seek failed.\r\n");
		return;
	}
	if( read(fd, buf, sizeof(buf)) == 0 ) {
		printf("[read_BPB] Read failed.\r\n");
		return;
	}
	
	// Contents dump
	for(i=0; i<sizeof(buf)/16; i++) {
		printf("%06X | %02X %02X %02X %02X %02X %02X %02X %02X   %02X %02X %02X %02X %02X %02X %02X %02X \r\n",
			i*0x10,
			buf[i*16+ 0], buf[i*16+ 1], buf[i*16+ 2], buf[i*16+ 3], 
			buf[i*16+ 4], buf[i*16+ 5], buf[i*16+ 6], buf[i*16+ 7], 
			buf[i*16+ 8], buf[i*16+ 9], buf[i*16+10], buf[i*16+11], 
			buf[i*16+12], buf[i*16+13], buf[i*16+14], buf[i*16+15] );
	}
	
	// Signature check
	if( buf[510] != 0x55 || buf[511] != 0xAA ) {
		printf("[read_BPB] BPB information is not valid.\r\n");
		return;
	}
	
	BPB_info.BytsPerSec   = buf[ 11] +   buf[ 12] * 0x100;
	BPB_info.ResvdSecCnt  = buf[ 14] +   buf[ 15] * 0x100;
	BPB_info.RootEntCnt   = buf[ 17] +   buf[ 18] * 0x100;
	BPB_info.FATSz16      = buf[ 22] +   buf[ 23] * 0x100;
	BPB_info.NumFATs      = buf[ 16];
	BPB_info.SecPerClus   = buf[ 13];
	BPB_info.TotSec16     = buf[ 19] +   buf[ 20] * 0x100;
	BPB_info.TotSec32     = buf[ 32]
						  + buf[ 33] * 0x100
						  + buf[ 34] * 0x10000
						  + buf[ 35] * 0x1000000;
						  
	if( BPB_info.FATSz16 == 0 ) {				// AREA Only for FAT32
		BPB_info.FATSz32 =  buf[ 36]
						+   buf[ 37] * 0x100
						+   buf[ 38] * 0x10000
						+   buf[ 39] * 0x1000000;
		BPB_info.RootClus = buf[ 44]
						+   buf[ 45] * 0x100
						+   buf[ 46] * 0x10000
						+   buf[ 47] * 0x1000000;
	}
	
	printf(" *** BPB information *** \r\n");
	printf("   BytsPerSec   : 0x%08X (%d)\r\n", BPB_info.BytsPerSec  , BPB_info.BytsPerSec  );
	printf("   ResvdSecCnt  : 0x%08X (%d)\r\n", BPB_info.ResvdSecCnt , BPB_info.ResvdSecCnt );
	printf("   RootEntCnt   : 0x%08X (%d)\r\n", BPB_info.RootEntCnt  , BPB_info.RootEntCnt  );
	printf("   FATSz16      : 0x%08X (%d)\r\n", BPB_info.FATSz16     , BPB_info.FATSz16     );
	printf("   NumFATs      : 0x%08X (%d)\r\n", BPB_info.NumFATs     , BPB_info.NumFATs     );
	printf("   SecPerClus   : 0x%08X (%d)\r\n", BPB_info.SecPerClus  , BPB_info.SecPerClus  );
	printf("   TotSec16     : 0x%08X (%d)\r\n", BPB_info.TotSec16    , BPB_info.TotSec16    );
	printf("   TotSec32     : 0x%08X (%d)\r\n", BPB_info.TotSec32    , BPB_info.TotSec32    );
	printf("   FATSz32      : 0x%08X (%d)\r\n", BPB_info.FATSz32     , BPB_info.FATSz32     );
	printf("   RootClus     : 0x%08X (%d)\r\n", BPB_info.RootClus    , BPB_info.RootClus     );
	
	printf("\r\n *** Calculated parameters *** \r\n");
	FATSectors			= BPB_info.FATSz16  ? BPB_info.FATSz16  : BPB_info.FATSz32;
	TotalSect			= BPB_info.TotSec16 ? BPB_info.TotSec16 : BPB_info.TotSec32;
	FirstDataSector		= BPB_info.ResvdSecCnt + (BPB_info.NumFATs * FATSectors) + RootDirSectors;
	DataSectors			= TotalSect - FirstDataSector;
	if( BPB_info.BytsPerSec != 0 ) {
		RootDirSectors		= ((BPB_info.RootEntCnt * 32) + (BPB_info.BytsPerSec - 1)) / BPB_info.BytsPerSec;
		CountofClusters		= DataSectors / BPB_info.SecPerClus;
	} else {
		RootDirSectors		= 0;
		CountofClusters		= 0;
	}
    printf("   RootDirSectors  : 0x%08X (%d)\r\n", RootDirSectors   ,  RootDirSectors   );
    printf("   FATSectors      : 0x%08X (%d)\r\n", FATSectors       ,  FATSectors       );
    printf("   FirstDataSector : 0x%08X (%d)\r\n", FirstDataSector  ,  FirstDataSector  );
    printf("   TotalSect       : 0x%08X (%d)\r\n", TotalSect        ,  TotalSect        );
    printf("   DataSectors     : 0x%08X (%d)\r\n", DataSectors      ,  DataSectors      );
    printf("   CountofClusters : 0x%08X (%d)\r\n", CountofClusters  ,  CountofClusters  );
	
	if( CountofClusters < 4085 ) {
		printf("This volume is FAT12\r\n");
	} else if( CountofClusters < 65525 ) {
		printf("This volume is FAT16\r\n");
	} else {
		printf("This volume is FAT32\r\n");
		IsFAT32 = 1;
	}
}

// ------------------------------------------------------
//  read file
// ------------------------------------------------------
unsigned char read_file( int fd, P_DIR_INFO pDirInfo, unsigned int index ) {
	int i;
	unsigned long read_size, current_cluster, current_sector;
	
	if( pDirInfo[index].FirstCluster == 0 ) {
		printf("[read_file] NO DATA.\r\n");
		return 0xFF;
	}
	
	if( index >  TotalFileNum) {
		printf("[read_file] Wrong index.\r\n");
		return 0xFF;
	}
	
	read_size = 0;
	current_cluster = pDirInfo[index].FirstCluster;
	current_sector = Cluster2Sector(current_cluster);
	move_window(fd, current_sector);
	
	printf("[read_file] FName:%12s  0x%08X %12d bytes\r\n", pDirInfo[index].Name, pDirInfo[index].FirstCluster, pDirInfo[index].FileSize);
	read_size = 0;
	while(1) 
	{
		// Contents dump
		for(i=0; i<sizeof(buf); i++) printf("%c",buf[i]);

		read_size += sizeof(buf);
		printf("\r\n(%d)\r\n", read_size);
		if( read_size >= pDirInfo[index].FileSize ) break;
		
		current_sector++;
		if( current_sector % BPB_info.SecPerClus == 0 ) {
			GetNextCluster(fd, &current_cluster);
			current_sector = Cluster2Sector(current_cluster);
		}
		move_window(fd, current_sector);
	}
	printf("\r\n[read_file] end of read file. read_size =%d(%d)\r\n", read_size, sizeof(buf));
}
// ------------------------------------------------------
//  Main function
// ------------------------------------------------------
main (int argc, char *argv[] ) {
	int fd, i;
	
	// Global initialize
	TotalFileNum = 0;
	IsFAT32 = 0;
	
	if( argc == 1 ) {
		printf("No file is specified.\r\n");
		return;
	}
	
	fd  = open(*++argv, O_RDONLY, 0);
	if(fd == -1) {
		printf("Can not open file.(%s)\r\n", *argv);
		return;
	}
	
	read_BPB(fd);
	
	init_dir_entry(fd, (P_DIR_ENTRY) &dir_entry, 0L );			// Set to root
	read_DIR(fd);
	sort_DIR(fd, (P_DIR_INFO)&dir_info);
	
	// for example, move to 2nd folder
	init_dir_entry(fd, (P_DIR_ENTRY) &dir_entry, dir_info[1].FirstCluster );			// Set to root
	read_DIR(fd);
	
	// for example, read 4th file
	read_file(fd, dir_info, 2);
}
